@@ -1,16 +1,14 @@
 import unittest
 
 from deployer.exceptions import ActionException, ExecCommandFailed
-from deployer.host_container import HostsContainer
 from deployer.inspection import Inspector
-from deployer.loggers import LoggerInterface
-from deployer.node import Node, SimpleNode, Env, alias
+from deployer.node import Node, SimpleNode, Env
 from deployer.node import map_roles, dont_isolate_yet, required_property, alias, IsolationIdentifierType
-from deployer.pseudo_terminal import Pty, DummyPty
 from deployer.query import Q
 
 from our_hosts import LocalHost, LocalHost1, LocalHost2, LocalHost3, LocalHost4, LocalHost5
 
+
 class NodeTest(unittest.TestCase):
     def test_assignments_to_node(self):
         """
@@ -25,7 +23,6 @@ class NodeTest(unittest.TestCase):
         MyNode.s = S2
         self.assertEqual(MyNode.s, S2) # TODO: the same for methods is not true!!!
 
-
     def test_node_initialisation(self):
         class S(Node):
             class Hosts:
@@ -57,8 +54,8 @@ class NodeTest(unittest.TestCase):
         self.assertEqual(s.T.hosts.roles, ['role1', 'role2'])
         self.assertEqual(s.U.hosts.roles, ['role1', 'role2'])
 
-        self.assertEqual(s.hosts, s.T.hosts)
-        self.assertEqual(s.hosts, s.U.hosts)
+        self.assertEqual(s.hosts.get_hosts_as_dict(), s.T.hosts.get_hosts_as_dict())
+        self.assertEqual(s.hosts.get_hosts_as_dict(), s.U.hosts.get_hosts_as_dict())
 
     def test_mapping(self):
         class S(Node):
@@ -66,11 +63,11 @@ class NodeTest(unittest.TestCase):
                 role1 = LocalHost
                 role2 = LocalHost2
 
-            @map_roles(role3='role1', role4='role2', role5='role3', role6=['role1', 'role2'])
+            @map_roles(role3='role1', role4='role2', role5='role3', role6=('role1', 'role2'))
             class T(Node):
                 pass
 
-            @map_roles(role7=['role1', 'role3'], role8='role1')
+            @map_roles(role7=('role1', 'role3'), role8='role1')
             class U(Node):
                 class V(Node):
                     pass
@@ -82,7 +79,7 @@ class NodeTest(unittest.TestCase):
                 class X(Node):
                     class Hosts:
                         role1 = LocalHost3
-                        role2 = [ LocalHost4, LocalHost5 ]
+                        role2 = { LocalHost4, LocalHost5 }
 
                 @map_roles(role11='role7')
                 class Y(Node):
@@ -91,13 +88,13 @@ class NodeTest(unittest.TestCase):
                         role1 = LocalHost1
 
         s = S()
-        self.assertEqual(isinstance(s, Node), True)
-        self.assertEqual(isinstance(s.T, Node), True)
-        self.assertEqual(isinstance(s.U, Node), True)
-        self.assertEqual(isinstance(s.U.V, Node), True)
-        self.assertEqual(isinstance(s.U.W, Node), True)
-        self.assertEqual(isinstance(s.U.X, Node), True)
-        self.assertEqual(isinstance(s.U.Y, Node), True)
+        self.assertIsInstance(s, Node)
+        self.assertIsInstance(s.T, Node)
+        self.assertIsInstance(s.U, Node)
+        self.assertIsInstance(s.U.V, Node)
+        self.assertIsInstance(s.U.W, Node)
+        self.assertIsInstance(s.U.X, Node)
+        self.assertIsInstance(s.U.Y, Node)
 
         self.assertEqual(s.hosts.roles, ['role1', 'role2'])
         self.assertEqual(s.T.hosts.roles, ['role3', 'role4', 'role5', 'role6'])
@@ -108,12 +105,18 @@ class NodeTest(unittest.TestCase):
         self.assertEqual(s.U.X.hosts.roles, ['role1', 'role2'])
         self.assertEqual(s.U.Y.hosts.roles, ['role11'])
 
-        self.assertEqual(s.T.hosts, HostsContainer({ 'role3': LocalHost, 'role4': LocalHost2, 'role5': [], 'role6': [ LocalHost, LocalHost2 ] }))
-        self.assertEqual(s.U.hosts, HostsContainer({ 'role7': LocalHost, 'role8': LocalHost }))
-        self.assertEqual(s.U.V.hosts, HostsContainer({ 'role7': LocalHost, 'role8': LocalHost }))
-        self.assertEqual(s.U.W.hosts, HostsContainer({ 'role9': [], 'role10': LocalHost }))
-        self.assertEqual(s.U.X.hosts, HostsContainer({ 'role1': LocalHost3, 'role2': [ LocalHost4, LocalHost5] }))
-        self.assertEqual(s.U.Y.hosts, HostsContainer({ 'role11': LocalHost }))
+        self.assertEqual(s.T.hosts.get_hosts_as_dict(),
+                { 'role3': {LocalHost}, 'role4': {LocalHost2}, 'role5': set(), 'role6': { LocalHost, LocalHost2 } })
+        self.assertEqual(s.U.hosts.get_hosts_as_dict(),
+                { 'role7': {LocalHost}, 'role8': {LocalHost} })
+        self.assertEqual(s.U.V.hosts.get_hosts_as_dict(),
+                { 'role7': {LocalHost}, 'role8': {LocalHost} })
+        self.assertEqual(s.U.W.hosts.get_hosts_as_dict(),
+                { 'role9': set(), 'role10': {LocalHost} })
+        self.assertEqual(s.U.X.hosts.get_hosts_as_dict(),
+                { 'role1': {LocalHost3}, 'role2': {LocalHost4, LocalHost5} })
+        self.assertEqual(s.U.Y.hosts.get_hosts_as_dict(),
+                { 'role11': {LocalHost} })
 
     def test_invalid_mapping(self):
         class NotANode(object): pass
@@ -138,7 +141,7 @@ class NodeTest(unittest.TestCase):
                 return self.hosts.filter('role1').run('/bin/echo echo', interactive=False)
 
             def echo_on_role2(self):
-                return self.hosts.get('role2').run('/bin/echo echo', interactive=False)
+                return self.hosts.filter('role2')[0].run('/bin/echo echo', interactive=False)
 
         s = S()
         env = Env(s)
@@ -169,7 +172,7 @@ class NodeTest(unittest.TestCase):
 
         try:
             env.return_false()
-        except ActionException, e:
+        except ActionException as e:
             self.assertIsInstance(e.inner_exception, ExecCommandFailed)
 
     def test_action_with_params(self):
@@ -215,7 +218,7 @@ class NodeTest(unittest.TestCase):
     def test_wrapping_middle_node_in_env(self):
         class S(Node):
             class Hosts:
-                role1 = LocalHost1, LocalHost2
+                role1 = { LocalHost1, LocalHost2 }
 
             class T(Node):
                 def action(self):
@@ -278,7 +281,7 @@ class NodeTest(unittest.TestCase):
     def test_simple_node(self):
         class N(SimpleNode):
             class Hosts:
-                host = LocalHost1, LocalHost2
+                host = { LocalHost1, LocalHost2 }
 
             def func(self):
                 return 'result'
@@ -290,7 +293,7 @@ class NodeTest(unittest.TestCase):
     def test_simple_node_getitem(self):
         class N(SimpleNode):
             class Hosts:
-                host = LocalHost1, LocalHost2
+                host = { LocalHost1, LocalHost2 }
 
             def func(self):
                 return 'result'
@@ -319,7 +322,7 @@ class NodeTest(unittest.TestCase):
         # __getitem__ should not be possible on a normal node.
         class N(Node):
             class Hosts:
-                host = LocalHost1, LocalHost2
+                host = { LocalHost1, LocalHost2 }
         n = N()
         self.assertRaises(KeyError, lambda:n[0]) # TODO: regex match: KeyError: __getitem__ on isolated node is not allowed.
 
@@ -328,7 +331,7 @@ class NodeTest(unittest.TestCase):
         # self.host as the index parameter.
         class Root(Node):
             class Hosts:
-                role = LocalHost1, LocalHost2
+                role = { LocalHost1, LocalHost2 }
 
             @map_roles('role')
             class A(SimpleNode.Array):
@@ -351,7 +354,7 @@ class NodeTest(unittest.TestCase):
 
         class A(Node):
             class Hosts:
-                my_role = LocalHost1, LocalHost2
+                my_role = { LocalHost1, LocalHost2 }
 
             @map_roles('my_role')
             class B(SimpleNode.Array):
@@ -376,7 +379,7 @@ class NodeTest(unittest.TestCase):
     def test_nested_simple_nodes(self):
         class N(SimpleNode):
             class Hosts:
-                host = LocalHost1, LocalHost2
+                host = { LocalHost1, LocalHost2 }
 
             class M(SimpleNode):
                 def func(self):
@@ -389,7 +392,7 @@ class NodeTest(unittest.TestCase):
     def test_simple_nodes_in_normal_node(self):
         class N(Node):
             class Hosts:
-                role1 = LocalHost1, LocalHost
+                role1 = { LocalHost1, LocalHost }
                 role2 = LocalHost3
 
             @map_roles('role1')
@@ -416,7 +419,7 @@ class NodeTest(unittest.TestCase):
     def test_calling_between_simple_and_normal_nodes(self):
         class N(Node):
             class Hosts:
-                role1 = LocalHost1, LocalHost
+                role1 = { LocalHost1, LocalHost }
                 role2 = LocalHost3
 
             def do_tests(this):
@@ -458,7 +461,7 @@ class NodeTest(unittest.TestCase):
 
         class N(Node):
             class Hosts:
-                role1 = LocalHost1, LocalHost2
+                role1 = { LocalHost1, LocalHost2 }
                 role2 = LocalHost3
 
             class M(Node):
@@ -509,9 +512,9 @@ class NodeTest(unittest.TestCase):
         # we will map *all* roles to 'host'
         class A(Node):
             class Hosts:
-                role1 = LocalHost1, LocalHost2
-                role2 = LocalHost3
-                role3 = LocalHost4
+                role1 = { LocalHost1, LocalHost2 }
+                role2 = { LocalHost3 }
+                role3 = { LocalHost4 }
 
             class B(SimpleNode.Array):
                 pass
@@ -523,8 +526,8 @@ class NodeTest(unittest.TestCase):
         # mapping between A and B, this hosts will be used.
         class A(Node):
             class Hosts:
-                role1 = LocalHost1, LocalHost2
-                role2 = LocalHost3, LocalHost4
+                role1 = { LocalHost1, LocalHost2 }
+                role2 = { LocalHost3, LocalHost4 }
 
             class B(SimpleNode.Array):
                 class Hosts:
@@ -547,7 +550,7 @@ class NodeTest(unittest.TestCase):
         # Exception: Invalid initialisation of SimpleNode.JustOne. 2 hosts given to <Node A.B>.
         class A(Node):
             class Hosts:
-                role1 = LocalHost1, LocalHost2
+                role1 = { LocalHost1, LocalHost2 }
 
             class B(SimpleNode.JustOne):
                 pass
@@ -608,7 +611,7 @@ class NodeTest(unittest.TestCase):
 
         try:
             env.action()
-        except ActionException, e:
+        except ActionException as e:
             self.assertIsInstance(e.inner_exception, AttributeError)
 
     def test_custom_node_init(self):
@@ -646,7 +649,7 @@ class NodeTest(unittest.TestCase):
         self.assertRaises(ActionException, env.action)
         try:
             env.action()
-        except ActionException, e:
+        except ActionException as e:
             self.assertIsInstance(e.inner_exception, NotImplementedError)
 
     def test_action_aliases(self):
@@ -671,11 +674,11 @@ class NodeTest(unittest.TestCase):
 
         class A(Node):
             class Hosts:
-                host = [ LocalHost1, LocalHost2, LocalHost3 ]
+                host = { LocalHost1, LocalHost2, LocalHost3 }
 
             class B(Node):
                 class Hosts:
-                    host = [ LocalHost4 ]
+                    host = { LocalHost4 }
 
             @map_roles('host')
             class C(SimpleNode.Array):
@@ -701,7 +704,7 @@ class NodeTest(unittest.TestCase):
     def test_default_action(self):
         class A(Node):
             class Hosts:
-                role = LocalHost1, LocalHost2
+                role = { LocalHost1, LocalHost2 }
 
             def __call__(self):
                 return 'A.call'
@@ -728,7 +731,7 @@ class NodeTest(unittest.TestCase):
 
         class A(Node):
             class Hosts:
-                role = LocalHost1, LocalHost2
+                role = { LocalHost1, LocalHost2 }
 
             @map_roles('role')
             class B(SimpleNode.Array):
@@ -754,7 +757,7 @@ class NodeTest(unittest.TestCase):
 
         class Root(Node):
             class Hosts:
-                host = LocalHost1, LocalHost2
+                host = { LocalHost1, LocalHost2 }
 
             @map_roles(host='host')
             class A(SimpleNode.Array):
@@ -784,8 +787,8 @@ class NodeTest(unittest.TestCase):
     def test_initialize_node(self):
         class A(Node):
             class Hosts:
-                role1 = LocalHost2
-                role2 = LocalHost4, LocalHost5
+                role1 = { LocalHost2 }
+                role2 = { LocalHost4, LocalHost5 }
 
             def action(self):
                 # Define a new Node-tree
@@ -810,8 +813,8 @@ class NodeTest(unittest.TestCase):
 
         class A(Node):
             class Hosts:
-                role1 = LocalHost1, LocalHost2, LocalHost3
-                role2 = LocalHost2, LocalHost4, LocalHost5
+                role1 = { LocalHost1, LocalHost2, LocalHost3 }
+                role2 = { LocalHost2, LocalHost4, LocalHost5 }
 
             @map_roles('role1', extra='role2')
             class B(SimpleNode.Array):
@@ -836,12 +839,11 @@ class NodeTest(unittest.TestCase):
     def test_nesting_normal_node_in_simple_node(self):
         # It is possible to nest multiple sequences of Node-SimpleNode.Array
         # inside each other. This behaves like a multi-dimensional array.
-        this = self
 
         class A(Node):
             class Hosts:
-                role1 = LocalHost1, LocalHost2, LocalHost3
-                role2 = LocalHost2, LocalHost4, LocalHost5
+                role1 = { LocalHost1, LocalHost2, LocalHost3 }
+                role2 = { LocalHost2, LocalHost4, LocalHost5 }
 
             @map_roles('role1', extra='role2')
             class B(SimpleNode.Array):
@@ -918,7 +920,7 @@ class NodeTest(unittest.TestCase):
         # host for this role.
         class A(Node):
             class Hosts:
-                role1 = LocalHost1, LocalHost2
+                role1 = { LocalHost1, LocalHost2 }
                 role2 = LocalHost3
 
             @map_roles('role2')
@@ -939,6 +941,8 @@ class NodeTest(unittest.TestCase):
 
             def action(self):
                 with self.hosts.cd('/tmp'):
+                    this.assertEqual(self.hosts.getcwd(), ['/tmp'])
+                    this.assertEqual(self.hosts[0].getcwd(), '/tmp')
                     this.assertEqual(self.hosts[0].run('pwd').strip(), '/tmp')
 
                     with self.hosts.cd('/'):
